
/**
 * Title:        "Knowledge Emerging from Behaviour" Architecture<p>
 * Description:  <p>
 * Copyright:    Copyright (c) Carlos Gershenson, C.Gershenson@sussex.ac.uk<p>
 * Company:      COGS, University of Sussex<p>
 * @author Carlos Gershenson, C.Gershenson@sussex.ac.uk
 * @version 3.0
 */
package keba;

import com.sun.j3d.utils.geometry.*;
import java.awt.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import keba.tools.*;

public abstract class Animat extends Phenomenon {
  AnimatFrame af;//set to null before saving...
  String name;
  float z2,x2;
  float th=0;//orientation angle (in radians)
  float noise=0.05f;//noise to be added to perceptions
  float rp=20; //radius of perception
  boolean verbose=true;
  boolean immortal=false;
  boolean dead=false;
  boolean leaveTrail=true;
  boolean toFile=false;//if true, text files record the state history of internal states...
  int periodWrite=50;//each when to write to file...
  int transWrite=0;//each when to write to file...
  StatStream ostream=null;
  public float energy=1;
  public float hunger=0;
  public float thirst=0;
  String msg;//set to null before saving...
  SBranchGroup trail, trailBG;
  STransformGroup trailTG, trailTG2;
  STransform3D trailT, trailT2;
  SAppearance trailApp;

  public Animat(float nz, float nx, float nradius, Color col, long nid,int type) {
    //z x radius colour flavourp flavourn odourp odourn hardness loudness id type
    super(nz,nx,nradius, col ,0.3f,0.0f,0.5f,0.0f,0.6f,0.3f, nid, type);
    name=String.valueOf(id);//initial name
    z2=z;
    x2=x;
    if (toFile){
      KEBA.iniStatFile("sims/"+name+KEBA.phType(type)+".txt", this.id);
    }
  }

  void init3D(){//can be overriden if you want diffeent shapes of animats...
    t=new STransform3D();
    t.rotY(th);
    t.setTranslation(new Vector3f(x,1+(0.5f*radius),z));
    t.setScale(new Vector3d(radius,radius,radius));
    tg.setTransform(t);
    TexturedAppearance tapp=new TexturedAppearance("keba/img/blueglisten.jpg");
    Color3f aColor    = new Color3f(colour);
    Color3f eColor    = new Color3f(colour);
    Color3f dColor    = new Color3f(colour);
    Color3f sColor    = new Color3f(colour);
    Material mat = new Material(aColor, eColor, dColor, sColor, 100.0f);
    mat.setLightingEnable(true);
    tapp.setMaterial(mat);
    //body
    Sphere body=new Sphere(1.0f,Primitive.GENERATE_NORMALS
                           | Primitive.GENERATE_TEXTURE_COORDS,tapp);
    TransformGroup bodyTG=new TransformGroup();
    Transform3D bodyT=new Transform3D();
    bodyT.setScale(new Vector3d(0.6,0.5,1.0));
    bodyTG.addChild(body);
    bodyTG.setTransform(bodyT);
    tg.addChild(bodyTG);
    //ears
    Cone ear1=new Cone(0.15f,0.3f,Primitive.GENERATE_NORMALS
                           | Primitive.GENERATE_TEXTURE_COORDS,tapp);
    TransformGroup ear1TG=new TransformGroup();
    Transform3D ear1T=new Transform3D();
    ear1T.rotX(3*Math.PI/16);
    ear1T.rotZ(-Math.PI/8);
    ear1T.setTranslation(new Vector3f(0.3f,0.35f,0.5f));
    ear1TG.addChild(ear1);
    ear1TG.setTransform(ear1T);
    tg.addChild(ear1TG);
    Cone ear2=new Cone(0.15f,0.3f,Primitive.GENERATE_NORMALS
                           | Primitive.GENERATE_TEXTURE_COORDS,tapp);
    TransformGroup ear2TG=new TransformGroup();
    Transform3D ear2T=new Transform3D();
    ear2T.rotX(3*Math.PI/16);
    ear2T.rotZ(Math.PI/8);
    ear2T.setTranslation(new Vector3f(-0.3f,0.35f,0.5f));
    ear2TG.addChild(ear2);
    ear2TG.setTransform(ear2T);
    tg.addChild(ear2TG);
    //nose
    Cone nose=new Cone(0.33f,0.66f,Primitive.GENERATE_NORMALS
                           | Primitive.GENERATE_TEXTURE_COORDS,tapp);
    TransformGroup noseTG=new TransformGroup();
    Transform3D noseT=new Transform3D();
    noseT.rotX(Math.PI/2);
    noseT.setTranslation(new Vector3f(0.0f,0.0f,1.0f));
    noseTG.addChild(nose);
    noseTG.setTransform(noseT);
    tg.addChild(noseTG);
    //tail
    Cone tail=new Cone(0.15f,1.75f,Primitive.GENERATE_NORMALS
                           | Primitive.GENERATE_TEXTURE_COORDS,tapp);
    TransformGroup tailTG=new TransformGroup();
    Transform3D tailT=new Transform3D();
    tailT.rotX(-5*Math.PI/8);
    tailT.setTranslation(new Vector3f(0.0f,-0.1f,-1.0f));
    tailTG.addChild(tail);
    tailTG.setTransform(tailT);
    tg.addChild(tailTG);
    //eyes
    Appearance eyeApp=new Appearance();
    Color3f eyeaColor    = new Color3f(0.15f, 0.0f, 0.05f);
    Color3f eyeeColor    = new Color3f(0.0f, 0.0f, 0.0f);
    Color3f eyedColor    = new Color3f(0.1f, 0.0f, 0.05f);
    Color3f eyesColor    = new Color3f(0.5f, 0.3f, 0.2f);
    Material eyeMat = new Material(eyeaColor, eyeeColor, eyedColor, eyesColor, 100.0f);
    eyeMat.setLightingEnable(true);
    eyeApp.setMaterial(eyeMat);
    Sphere eye1=new Sphere(0.1f,Primitive.GENERATE_NORMALS,eyeApp);
    TransformGroup eye1TG=new TransformGroup();
    Transform3D eye1T=new Transform3D();
    eye1T.setTranslation(new Vector3f(0.22f,0.2f,0.8f));
    eye1TG.addChild(eye1);
    eye1TG.setTransform(eye1T);
    tg.addChild(eye1TG);
    Sphere eye2=new Sphere(0.1f,Primitive.GENERATE_NORMALS,eyeApp);
    TransformGroup eye2TG=new TransformGroup();
    Transform3D eye2T=new Transform3D();
    eye2T.setTranslation(new Vector3f(-0.22f,0.2f,0.8f));
    eye2TG.addChild(eye2);
    eye2TG.setTransform(eye2T);
    tg.addChild(eye2TG);

    if (leaveTrail)
      iniTrail();
  }

  public void iniTrail(){
    trailApp=new SAppearance();
    Color3f trColour    = new Color3f(colour);
    trailApp.setMaterial(new Material(trColour, trColour, trColour, trColour, 100.0f));
    trail=new SBranchGroup();
    trail.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
    trail.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
    trail.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
    trail.setCapability(BranchGroup.ALLOW_DETACH);
    KEBA.scene.addChild(trail);
  }

  void clean(int mode){
    if (mode==KEBA.RESET){
      removeTrail();
    }
    msg="";
    af.dispose();
    af=null;
    z2=z; x2=x;
  }

  public void destroy(){
    removeTrail();
    super.destroy();
  }

  abstract void restore();
    // ( just af=new AnimatXXXXFrame(this); )

  abstract void step();
    // (implement stepState() at the end...)

  void stepState() {//for all animats
    if (!dead){
      //do torus
      boolean didTorus=false;
      if (z>KEBA.envRadius){ z=-KEBA.envRadius; didTorus=true;}
      if (z<-KEBA.envRadius){ z=KEBA.envRadius; didTorus=true;}
      if (x>KEBA.envRadius){ x=-KEBA.envRadius; didTorus=true;}
      if (x<-KEBA.envRadius){ x=KEBA.envRadius; didTorus=true;}
      //update position in VirtualUniverse
      t.rotY(th);
      t.setTranslation(new Vector3f(x,1+(0.5f*radius),z));
      t.setScale(new Vector3d(radius,radius,radius));
      tg.setTransform(t);

      if ((leaveTrail)&&(!didTorus)){
        trailBG=new SBranchGroup();
        trailTG=new STransformGroup();
        trailT=new STransform3D();
        trailTG2=new STransformGroup();
        trailT2=new STransform3D();
        Cylinder trailSeg=new Cylinder(0.1f,
                                (float)Math.sqrt(Math.pow(z-z2,2)+Math.pow(x-x2,2)),
                                Primitive.GENERATE_NORMALS,trailApp);
        trailT2.rotX(Math.PI/2);
        trailTG2.setTransform(trailT2);
        trailTG2.addChild(trailSeg);
        trailT.rotY(Math.atan((x-x2)/(z-z2)));
        trailT.setTranslation(new Vector3f((x+x2)/2,1+(0.5f*radius),(z2+z)/2));
        trailTG.setTransform(trailT);
        trailTG.addChild(trailTG2);
        trailBG.addChild(trailTG);
        trail.addChild(trailBG);
      }
      z2=z;
      x2=x;

      thirst+= 0.0002;
      hunger+= 0.00012;
      if(thirst < 0.2) {
        energy+= 0.002;
      }
      if(hunger < 0.2) {
        energy+= 0.002;
      }
      if(thirst > 0.8) {
        if (!immortal)
          energy-= 0.0002;
      }
      if(hunger > 0.8) {
        if (!immortal)
          energy-= 0.0002;
      }
      if (!immortal){
        float id_d[]=KEBA.closest(this,KEBA.LIGHTNING);
        if (id_d[1]<=0){//touching
          energy-=0.2;
        }
        id_d=KEBA.closest(this,KEBA.ROCK);
        if (id_d[1]<=0){//touching
          energy-=0.05;
        }
        if (energy<=0){
          dead=true;
        }
      }
      //bound internal states
      if (energy < 0)energy = 0;
      if (energy > 1)energy = 1;
      if (thirst < 0)thirst = 0;
      if (thirst > 1)thirst = 1;
      if (hunger < 0)hunger = 0;
      if (hunger > 1)hunger = 1;
      if (th<0) th+=2*(float)Math.PI;
      if (th!=0) th=th%(2*(float)Math.PI);

      if ((toFile)&&((time%periodWrite)==transWrite)){
        String str = Tools.str(time)+"\t"+
                     Tools.str(energy,100)+"\t"+
                     Tools.str(thirst,100)+"\t"+
                     Tools.str(hunger,100)+"\n";
        KEBA.doStats(str, this.id);
      }

      af.step();
      time++;
    }
    if (energy>0) dead=false;
  }

  public void removeTrail(){
    trail.detach();
    if (leaveTrail)
      iniTrail();
  }

  float noise(){
    return (KEBA.rnd.nextFloat()*this.noise)-(this.noise/2);
  }

  public void setEnergy(float ne){
    this.energy=ne;
  }

  public void setThirst(float n){
    this.thirst=n;
  }

  public void setHunger(float n){
    this.hunger=n;
  }

  public void setZ(float n){
    this.z=n;
  }

  public void setX(float n){
    this.x=n;
  }

  public void setTh(float n){
    this.th=n;
  }

  double getToroidalD(Phenomenon ph){
    double minD=KEBA.envRadius*2*Math.sqrt(2);
    double d;
    for (int i=-(KEBA.envRadius*2);i<=(KEBA.envRadius*2);i+=(KEBA.envRadius*2)){
      for (int j=-(KEBA.envRadius*2);j<=(KEBA.envRadius*2);j+=(KEBA.envRadius*2)){
        d=Math.sqrt(Math.pow(this.z-ph.z+i,2)+Math.pow(this.x-ph.x+j,2))
          -this.radius-ph.radius;
        //System.out.println("d: "+d+" i:"+i+" j:"+j);
        if (d<minD)
          minD=d;
      }
    }
    //System.out.println("minD: "+minD);
    return minD;
  }

}