
/**
 * Title:        "Knowledge Emerging from Behaviour" Archistecture<p>
 * Description:  <p>
 * Copyright:    Copyright (c) Carlos Gershenson, C.Gershenson@sussex.ac.uk<p>
 * Company:      COGS, University of Sussex<p>
 * @author Carlos Gershenson, C.Gershenson@sussex.ac.uk
 * @version 3.0
 */
package keba;

import java.awt.Color;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.*;
import keba.tools.*;

public class AnimatBrait extends Animat {
  float noise=0.5f;//pref. between 0 and 1... but of course... if you're perverse enought... try it!
  //directions
  public static final int LEFT=-1;
  public static final int RIGHT=1;

  public AnimatBrait(float nz, float nx, float nradius, long nid) {
    super(nz,nx,nradius,new Color(0.8f, 0.95f, 1.0f), nid,KEBA.ANIMAT_BRAIT);
    init3D();
    //create AnimatFrame
    af=new AnimatBraitFrame(this);
  }

  void restore(){
    af=new AnimatBraitFrame(this);
  }

  void step() {
    if (!dead){
      msg="+ Animat "+name+" at: "+Tools.str(z)+", "+Tools.str(x)+
          ", angle: "+Tools.str(th)+", t: "+Tools.str(time)+"\n";
      move();
    }
    stepState();
  }

  public void move(){//ASM???
    int perc;

    //noise...
    rightMotor(noise*(Math.PI/20)*energy*(hunger+thirst)*KEBA.rnd.nextFloat());
    leftMotor(noise*(Math.PI/20)*energy*(hunger+thirst)*KEBA.rnd.nextFloat());
    //the higher the noise the higher the switching in behaviours!

    perc=rainPerc();
    if (perc==LEFT){//approach
      rightMotor((Math.PI/4)*energy*thirst);
    } else if (perc==RIGHT){
      leftMotor((Math.PI/4)*energy*thirst);
    }

    perc=foodPerc();
    if (perc==LEFT){//approach
      rightMotor((Math.PI/4)*energy*hunger);
    } else if (perc==RIGHT){
      leftMotor((Math.PI/4)*energy*hunger);
    }

    perc=rockNear();
    if (perc==LEFT){//avoid
      leftMotor((Math.PI/4)*energy);
    } else if (perc==RIGHT){
      rightMotor((Math.PI/4)*energy);
    }

    if ((foodAtRange())&&(hunger>0.0)){
      eat();
    }
    if ((rainAtRange())&&(thirst>0.0)){
      drink();
    }
  }

//perceptual functions
  public int LRPh(Phenomenon ph){//checks if ph is left or right of animat...
    if ( ( ((th<(Math.PI/2))||(th>=(3*Math.PI/2)))&&( ph.x>=x+(Math.tan(th)*(ph.z-z)) ) ) ||
         ( ((th>=(Math.PI/2))&&(th<(3*Math.PI/2)))&&( ph.x<x+(Math.tan(th)*(ph.z-z)) ) ) ){
      return LEFT;
    }else{
      return RIGHT;
    }
  }

  public int foodPerc(){
    float id_d[]=KEBA.maxpond(this,KEBA.FOOD);
    if (id_d[1]<=rp){
      Phenomenon ph=KEBA.getPhenomenon((long)id_d[0]);
      return LRPh(ph);
    }
    return 0;//none
  }
  public int rainPerc(){
    float id_d[]=KEBA.maxpond(this,KEBA.RAIN);
    if (id_d[1]<=rp){
      Phenomenon ph=KEBA.getPhenomenon((long)id_d[0]);
      return LRPh(ph);
    }
    return 0;//none
  }
  public int rockNear(){
    float id_d[]=KEBA.maxpond(this,KEBA.ROCK);
    if (id_d[1]<=(radius*2)){
      Phenomenon ph=KEBA.getPhenomenon((long)id_d[0]);
      return LRPh(ph);
    }
    return 0;//none
  }
  public boolean foodAtRange(){
    float id_d[]=KEBA.closest(this,KEBA.FOOD);
    if (id_d[1]<=0){
      return true;
    }
    return false;
  }
  public boolean rainAtRange(){
    Phenomenon ph;
    float id_d[]=KEBA.closest(this,KEBA.RAIN);
    if (id_d[1]<=0){
      return true;
    }
    return false;
  }


//motors
  public void rightMotor(double thr) {//from ASIA... turn left
    float h = z- (float) Math.cos(th);
    float k = x- (float) Math.sin(th);
    z = (float) h+(float) Math.cos(th+thr);
    x = (float) k+(float) Math.sin(th+thr);
    th += thr;
  }
  public void leftMotor(double thr) {//from ASIA... turn right
    float h = z- (float) Math.cos(th+(Math.PI/2));
    float k = x- (float) Math.sin(th+(Math.PI/2));
    z = (float) h+(float) Math.cos(th+(Math.PI/2)-thr);
    x = (float) k+(float) Math.sin(th+(Math.PI/2)-thr);
    th -= thr;
  }

  public void eat(){
    Phenomenon ph, nph;
    float id_d[]=KEBA.closest(this,KEBA.FOOD);
    ph=KEBA.getPhenomenon((long)id_d[0]);
    if (ph.radius>0.1){
      nph=new Food(ph.z,ph.x,ph.radius-0.1f,ph.id);
      KEBA.addPhenomenon(nph, false);
    }
    KEBA.removePhenomenon(ph, false);
    hunger-=0.1;
  }
  public void drink(){
    thirst-=0.1;
  }
}