
/**
 * Title:        "Knowledge Emerging from Behaviour" Architecture<p>
 * Description:  <p>
 * Copyright:    Copyright (c) Carlos Gershenson, C.Gershenson@sussex.ac.uk<p>
 * Company:      COGS, University of Sussex<p>
 * @author Carlos Gershenson, C.Gershenson@sussex.ac.uk
 * @version 3.0
 */
package keba;

import java.awt.Color;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.*;
import keba.tools.*;

public class AnimatKEBA extends Animat {
  float[] sens;//inputs from sensors
  int behaviour=3;//this should be changed with a proper ASM...
  KEB keb;
  int maxlevs=2;//maximun KEB levels
  int maxkons=50;//maximum koncepts
  int maxpersist=2000;//maximun persistance of unused koncepts
  int kons=0;//number of koncepts in animat
  boolean dokeb=true;
  boolean introcept=false;//perceive internal medium
  boolean propiocept=false;//perceive actions
  public float satisf=0;//satisfaction
  public float cumsatisf=0;//cummulative satisfaction
  public int action=0;
  Concept c;
  //CONSTANTS
  //preconcepts
  public static final int TYPES_OF_SENSOR=20;
  public static final int REDNESS=0;
  public static final int GREENNESS=1;
  public static final int BLUENESS=2;
  public static final int PFLAVOUR=3;
  public static final int NFLAVOUR=4;
  public static final int PODOUR=5;
  public static final int NODOUR=6;
  public static final int HARDNESS=7;
  public static final int LOUDNESS=8;
  //need to do all sensors first, then internal variables. If to add more sensors, renumber
  public static final int ENERGY=9;
  public static final int THIRST=10;
  public static final int HUNGER=11;
  //after internal variables, list actions
  public static final int NOACT=12;
  public static final int EXPLORING=13;
  public static final int EATING=14;
  public static final int DRINKING=15;
  public static final int APPROACHING_FOOD=16;
  public static final int APPROACHING_RAIN=17;
  public static final int APPROACHING_LIGHTNING=18;
  public static final int AVOIDING=19;

  public static final int NINT=3;//number of internal variables
  //actions
  public static final int NACT=8;//number of actions
  public static final int NONE=0;
  public static final int EXPLORE=1;
  public static final int EAT=2;
  public static final int DRINK=3;
  public static final int APPROACH_FOOD=4;
  public static final int APPROACH_RAIN=5;
  public static final int APPROACH_LIGHTNING=6;
  public static final int AVOID=7;
  //misc
  public static final float YUD=2;//importance of previous experiences for cummulative satisfaction

  public AnimatKEBA(float nz, float nx, float nradius, long nid) {
    super(nz,nx,nradius,new Color(0.09f, 0.13f, 0.65f), nid,KEBA.ANIMAT_KEBA);
    init3D();
    //create AnimatFrame
    af=new AnimatKEBAFrame(this);
    //init koncept counter
    kons=0;
    //initialize keb
    keb=new KEB(this,0,maxlevs,maxkons,maxpersist,noise);//generate zero level concept vector
    c=new Concept(sensType(REDNESS),0,0); keb.addKoncept(c);
    c=new Concept(sensType(GREENNESS),0,0); keb.addKoncept(c);
    c=new Concept(sensType(BLUENESS),0,0); keb.addKoncept(c);
    c=new Concept(sensType(PFLAVOUR),0,0); keb.addKoncept(c);
    c=new Concept(sensType(NFLAVOUR),0,0); keb.addKoncept(c);
    c=new Concept(sensType(PODOUR),0,0); keb.addKoncept(c);
    c=new Concept(sensType(NODOUR),0,0); keb.addKoncept(c);
    c=new Concept(sensType(HARDNESS),0,0); keb.addKoncept(c);
    c=new Concept(sensType(LOUDNESS),0,0); keb.addKoncept(c);
    if (introcept){
      c=new Concept(sensType(ENERGY),0,0); keb.addKoncept(c);
      c=new Concept(sensType(THIRST),0,0); keb.addKoncept(c);
      c=new Concept(sensType(HUNGER),0,0); keb.addKoncept(c);
    }
    if (propiocept){
      c=new Concept(sensType(NOACT),0,0); keb.addKoncept(c);
      c=new Concept(sensType(EXPLORING),0,0); keb.addKoncept(c);
      c=new Concept(sensType(EATING),0,0); keb.addKoncept(c);
      c=new Concept(sensType(DRINKING),0,0); keb.addKoncept(c);
      c=new Concept(sensType(APPROACHING_FOOD),0,0); keb.addKoncept(c);
      c=new Concept(sensType(APPROACHING_RAIN),0,0); keb.addKoncept(c);
      c=new Concept(sensType(APPROACHING_LIGHTNING),0,0); keb.addKoncept(c);
      c=new Concept(sensType(AVOIDING),0,0); keb.addKoncept(c);
     }
  }

  public static String sensType(int type){
    if (type==REDNESS) return new String("Redness");
    if (type==GREENNESS) return new String("Greenness");
    if (type==BLUENESS) return new String("Blueness");
    if (type==PFLAVOUR) return new String("Flavour+");
    if (type==NFLAVOUR) return new String("Flavour-");
    if (type==PODOUR) return new String("Odour+");
    if (type==NODOUR) return new String("Odour-");
    if (type==HARDNESS) return new String("Hardness");
    if (type==LOUDNESS) return new String("Loudness");
    if (type==ENERGY) return new String("Energy");
    if (type==THIRST) return new String("Thirst");
    if (type==HUNGER) return new String("Hunger");
    if (type==NOACT) return new String("No Action");
    if (type==EXPLORING) return new String("Exploring");
    if (type==EATING) return new String("Eating");
    if (type==DRINKING) return new String("Drinking");
    if (type==APPROACHING_FOOD) return new String("Approaching Food");
    if (type==APPROACHING_RAIN) return new String("Approaching Rain");
    if (type==APPROACHING_LIGHTNING) return new String("Approaching Lightning");
    if (type==AVOIDING) return new String("Avoiding");
    return new String("Unknown sensation");
  }

  public static String actType(int type){
    return sensType(type+TYPES_OF_SENSOR-NACT)+ "... ";
  }

  public boolean needsTouch(int type){
    if ((type==PFLAVOUR)||(type==NFLAVOUR)||(type==HARDNESS))
      return true;
    else
      return false;
  }

  public void restore(){
    af=new AnimatKEBAFrame(this);
  }

  public void step() {
    float[] acts;
    satisf=0;
    if (!dead){
      msg="\n+ Animat "+name+" at: "+Tools.str(z)+", "+Tools.str(x)+
          ", angle: "+Tools.str(th)+", t: "+Tools.str(time)+"\n"+
          "koncepts: "+Tools.str(kons)+"\n";
      if (dokeb){
        perceive();
      }
      //do behaviours... (should change this for a proper ASM or BPS...)
      /*switch(behaviour){
        case 0://wander
          th+=(KEBA.rnd.nextFloat()*Math.PI/4)-Math.PI/8;//get random angle
          z+=Math.cos(th)*this.radius;//just follow the angle
          x+=Math.sin(th)*this.radius;//just follow the angle
        break;
        case 1://circular
          th+=Math.PI/20;//do a circle each 20 steps
          z+=Math.cos(th)*this.radius;//step accordingly to your size...
          x+=Math.sin(th)*this.radius;
        break;
        case 2://linear
          z+=Math.cos(th)*this.radius;//step accordingly to your size...
          x+=Math.sin(th)*this.radius;
        break;
        case 3://static
          //...que??
        break;
      }*/

      if (dokeb){
        acts=keb.getActions();
        if (Tools.getMax(acts)>0){
          action=Tools.getMax(acts);
        }else{
          action=EXPLORE;
        }
      }else{
        action=(int)Math.floor(KEBA.rnd.nextFloat()*NACT);
      }
      switch(action){
        case NONE:
          satisf+=noact();
        break;
        case EXPLORE:
          satisf+=explore();
        break;
        case EAT:
          satisf+=eat();
          hunger-=satisf;
        break;
        case DRINK:
          satisf+=drink();
          thirst-=satisf;
        break;
        case APPROACH_FOOD:
          satisf+=approachFood();
        break;
        case APPROACH_RAIN:
          satisf+=approachRain();
        break;
        case APPROACH_LIGHTNING:
          satisf+=approachLightning();
        break;
        case AVOID:
          satisf+=avoid();
        break;
        default:
        break;
      }

      satisf-=0.00001;
      if(thirst < 0.2) {
        if (energy<1)
          satisf+= 0.002;
      }
      if(hunger < 0.2) {
        if (energy<1)
          satisf+= 0.002;
      }
      if(thirst > 0.8) {
        satisf-= 0.0002;
      }
      if(hunger > 0.8) {
        satisf-= 0.0002;
      }

      cumsatisf=(satisf+(cumsatisf*YUD))/(YUD+1);

      msg+="Satisfaction: "+Tools.str(satisf)+
           " Cummulative: "+Tools.str(cumsatisf)+"\n";
      msg+=actType(action)+"\n";
      if ((dokeb)&&(satisf!=0)){
        keb.linkConcAct(action, satisf);
        //keb.linkConcAct(action, cumsatisf);
      }

    }
    stepState();
  }

  public void inckons(){
    kons++;
  }
  public void deckons(){
    kons--;
  }

  public void perceive(){
    sens=new float[TYPES_OF_SENSOR];
    int phenPerc=0;//count how many phenomena were perceived this sim step
    int phenTouch=0;//count how many phenomena were touched this sim step
    Phenomenon ph;
    double d;//distance between animat and phenomena
    float colour;
    for (int i=0;i<TYPES_OF_SENSOR;i++)
      sens[i]=noise();//add noise to sensors
    if (introcept){
      sens[ENERGY]+=energy;
      sens[THIRST]+=thirst;
      sens[HUNGER]+=hunger;
      for (int i=(TYPES_OF_SENSOR-NACT-NINT);i<(TYPES_OF_SENSOR-NACT);i++){
        keb.updateElementAt(sens[i],i);
      }
    }
    if (propiocept){
      switch(action){
        case NONE:
          sens[NOACT]+=1;
        break;
        case EXPLORE:
          sens[EXPLORING]+=1;
        break;
        case EAT:
          sens[EATING]+=1;
        break;
        case DRINK:
          sens[DRINKING]+=1;
        break;
        case APPROACH_FOOD:
          sens[APPROACHING_FOOD]+=1;
        break;
        case APPROACH_RAIN:
          sens[APPROACHING_RAIN]+=1;
        break;
        case APPROACH_LIGHTNING:
          sens[APPROACHING_LIGHTNING]+=1;
        break;
        case AVOID:
          sens[AVOIDING]+=1;
        break;
      }
      for (int i=(TYPES_OF_SENSOR-NACT);i<TYPES_OF_SENSOR;i++){
        keb.updateElementAt(sens[i],i);
      }
    }

    keb.delIniMsg();
    for (int i=0;i<KEBA.phenomena.size();i++){
      ph=(Phenomenon)KEBA.phenomena.elementAt(i);
      if (ph.id!=this.id){//don't perceive yourself
        d=getToroidalD(ph);
        //msg+="d: "+Tools.str((float)d)+"\n";
        if(d<=rp){//phenomenon perceived
          for (int j=0;j<(TYPES_OF_SENSOR-NACT-NINT);j++){
            sens[j]=noise();//add noise to sensors
          }
          phenPerc++;
          colour=ph.colour.getRed();
          colour/=256;
          sens[REDNESS]+=(colour);
          colour=ph.colour.getGreen();
          colour/=256;
          sens[GREENNESS]+=(colour);
          colour=ph.colour.getBlue();
          colour/=256;
          sens[BLUENESS]+=(colour);
          sens[PODOUR]+=ph.odour[0];
          sens[NODOUR]+=ph.odour[1];
          sens[LOUDNESS]+=ph.loudness;
          if(d<=0){//phenomenon touching animat
            phenTouch++;
            sens[PFLAVOUR]+=ph.flavour[0];
            sens[NFLAVOUR]+=ph.flavour[1];
            sens[HARDNESS]+=ph.hardness;
          }
          else if ((ph.type==KEBA.ROCK)||(ph.type==KEBA.LIGHTNING)){
            if (d<=radius){
              phenTouch++;
              sens[PFLAVOUR]+=ph.flavour[0];
              sens[NFLAVOUR]+=ph.flavour[1];
              sens[HARDNESS]+=ph.hardness;
            }
          }
          for (int j=0;j<(TYPES_OF_SENSOR-NACT-NINT);j++){
            keb.updateElementAt(sens[j],j);
          }
          keb.writeIniMsg();
          keb.doAbstraction(time, KEB.INI);//abstract only phenomena (level 1)
        }//d<rp
      }//!=id
    }//for... phenomena

//    msg+="  SENSORS (averages:):\n";
//    for (int i=0;i<(TYPES_OF_SENSOR-NACT-NINT);i++){
//      if (needsTouch(i)){
//        if (phenTouch>0)
//          sens[i]/=phenTouch;
//      } else {
//        if (phenPerc>0)
//          sens[i]/=phenPerc;
//      }
//      msg+=sensType(i)+": "+Tools.str(sens[i])+"\n";
//    }
//    if (introcept){
//      for (int i=(TYPES_OF_SENSOR-NACT-NINT);i<(TYPES_OF_SENSOR-NACT);i++){
//        msg+=sensType(i)+": "+Tools.str(sens[i])+"\n";
//      }
//    }
//    if (propiocept){
//      for (int i=(TYPES_OF_SENSOR-NACT);i<TYPES_OF_SENSOR;i++){
//        msg+=sensType(i)+": "+Tools.str(sens[i])+"\n";
//      }
//    }

    keb.doAbstraction(time, KEB.DEL);//abstract!
    if (keb.meta!=null){
      keb.meta.updateZeroesLev1(time);
    }
    msg+=keb.getMessage();
  }



//REINFORCEMENT FUNCTIONS
  public float noact(){
    Phenomenon ph;
    float satisf=0;
    float id_d[]=KEBA.closest(this,KEBA.FOOD);
    if (id_d[1]<=rp){
      satisf-=0.2*hunger;//for not eating it!
    }
    id_d=KEBA.closest(this,KEBA.ROCK);
    if (id_d[1]<=radius) satisf-=0.4;//for not avoiding it...
    id_d=KEBA.closest(this,KEBA.RAIN);
    if (id_d[1]<=rp) satisf-=0.2*thirst;//for not drinking from it!
    id_d=KEBA.closest(this,KEBA.LIGHTNING);
    if (id_d[1]<=radius) satisf-=0.5;//for not moving!
    //no reinforcement for spots
    //no reinf. for animats
    satisf+=0.1*(0.2-hunger);
    satisf+=0.1*(0.2-thirst);
    return satisf;
  }

  public float explore(){
    float satisf=0;
    randommove();
    if ((hunger>0.1)||(thirst>0.1)) satisf+=0.1;//ojo
    float id_d[]=KEBA.closest(this,KEBA.RAIN);
    if (id_d[1]<=rp){
      satisf-=0.4*thirst;
    }
    id_d=KEBA.closest(this,KEBA.FOOD);
    if (id_d[1]<=rp){
      satisf-=0.4*hunger;
    }
    id_d=KEBA.closest(this,KEBA.ROCK);
    if (id_d[1]<=radius){
      satisf-=0.5;
    }
    id_d=KEBA.closest(this,KEBA.LIGHTNING);
    if (id_d[1]<=radius){
      satisf-=0.6;
    }
    return satisf;
  }

  public float eat(){
    Phenomenon ph,nph;
    Animat an;
    float satisf=0;
    float id_d[]=KEBA.closest(this,KEBA.FOOD);
    if (id_d[1]<=0){//touching food
      ph=KEBA.getPhenomenon((long)id_d[0]);
      if (ph.radius>0.1){
        nph=new Food(ph.z,ph.x,ph.radius-0.1f,ph.id);
        KEBA.addPhenomenon(nph, false);
      }
      KEBA.removePhenomenon(ph, false);
      satisf+=0.2*hunger;
    } else {
      id_d=KEBA.closest(this,KEBA.ANIMAT_KEBA);
      if (id_d[1]<=0){//touching Animat
        an=(Animat)KEBA.getPhenomenon((long)id_d[0]);
        if (an.energy>0){
          an.setEnergy(an.energy-0.1f);
          satisf+=0.2*hunger;//cannibalism!
        }
      }
      id_d=KEBA.closest(this,KEBA.RAIN);
      if (id_d[1]<=0)satisf-=0.01;
      id_d=KEBA.closest(this,KEBA.ROCK);
      if (id_d[1]<=0){//touching rock
        ph=KEBA.getPhenomenon((long)id_d[0]);
        if (ph.radius>0.1){
          nph=new Rock(ph.z,ph.x,ph.radius-0.1f,ph.id);
          KEBA.addPhenomenon(nph, false);
        }
        KEBA.removePhenomenon(ph, false);
        satisf-=0.4;
      } else if (id_d[1]<=radius){
        satisf-=0.4;
      }
      id_d=KEBA.closest(this,KEBA.LIGHTNING);
      if (id_d[1]<=radius){//touching lightning
        satisf-=0.5;
      }
      id_d=KEBA.closest(this,KEBA.SPOT_RED);
      if (id_d[1]<=0){
        ph=KEBA.getPhenomenon((long)id_d[0]);
        if (ph.radius>0.1){
          nph=new SpotRed(ph.z,ph.x,ph.radius-0.1f,ph.id);
          KEBA.addPhenomenon(nph, false);
        }
        KEBA.removePhenomenon(ph, false);
        satisf-=0.3;
      }
      id_d=KEBA.closest(this,KEBA.SPOT_GREEN);
      if (id_d[1]<=0) {
        ph=KEBA.getPhenomenon((long)id_d[0]);
        if (ph.radius>0.1){
          nph=new SpotGreen(ph.z,ph.x,ph.radius-0.1f,ph.id);
          KEBA.addPhenomenon(nph, false);
        }
        KEBA.removePhenomenon(ph, false);
        satisf-=0.01;
      }
      id_d=KEBA.closest(this,KEBA.SPOT_BLUE);
      if (id_d[1]<=0){
        ph=KEBA.getPhenomenon((long)id_d[0]);
        if (ph.radius>0.1){
          nph=new SpotBlue(ph.z,ph.x,ph.radius-0.1f,ph.id);
          KEBA.addPhenomenon(nph, false);
        }
        KEBA.removePhenomenon(ph, false);
        satisf-=0.1;
      }
      id_d=KEBA.closest(this,KEBA.SPOT_YELLOW);
      if (id_d[1]<=0) {
        ph=KEBA.getPhenomenon((long)id_d[0]);
        if (ph.radius>0.1){
          nph=new SpotYellow(ph.z,ph.x,ph.radius-0.1f,ph.id);
          KEBA.addPhenomenon(nph, false);
        }
        KEBA.removePhenomenon(ph, false);
        satisf-=0.1;
      }
      id_d=KEBA.closest(this,KEBA.SPOT_WHITE);
      if (id_d[1]<=0) {
        ph=KEBA.getPhenomenon((long)id_d[0]);
        if (ph.radius>0.1){
          nph=new SpotWhite(ph.z,ph.x,ph.radius-0.1f,ph.id);
          KEBA.addPhenomenon(nph, false);
        }
        KEBA.removePhenomenon(ph, false);
        satisf-=0.2;
      }
      id_d=KEBA.closest(this,KEBA.SPOT_BLACK);
      if (id_d[1]<=0) {
        ph=KEBA.getPhenomenon((long)id_d[0]);
        if (ph.radius>0.1){
          nph=new SpotBlack(ph.z,ph.x,ph.radius-0.1f,ph.id);
          KEBA.addPhenomenon(nph, false);
        }
        KEBA.removePhenomenon(ph, false);
        satisf-=0.1;
      }
      id_d=KEBA.closest(this,KEBA.SPOT_RND);
      if (id_d[1]<=0) {
        ph=KEBA.getPhenomenon((long)id_d[0]);
        if (ph.radius>0.1){
          nph=new SpotRnd(ph.z,ph.x,ph.radius-0.1f,ph.id);
          KEBA.addPhenomenon(nph, false);
        }
        KEBA.removePhenomenon(ph, false);
        satisf-=0.1;
      }
      satisf-=0.02;//not to eat air...
    }
    return satisf;
  }

  public float drink(){
    Phenomenon ph;
    float satisf=0;
    float id_d[]=KEBA.closest(this,KEBA.RAIN);
    if (id_d[1]<=0){//touching rain
        //don't change Rain
      satisf+=0.25*thirst;
    }else{
      satisf-=0.02;
    }
    id_d=KEBA.closest(this,KEBA.ROCK);
    if (id_d[1]<=radius){
      satisf-=0.5;
    }
    id_d=KEBA.closest(this,KEBA.LIGHTNING);
    if (id_d[1]<=radius){
      satisf-=0.6;
    }
    return satisf;
  }

  public float approachFood(){
    float satisf=0;
    float id_d[]=KEBA.closest(this,KEBA.FOOD);
    if ((id_d[1]>0)&&(id_d[1]<=rp)){
      satisf+=0.2*hunger;
      approach(KEBA.FOOD);
    } else{
      satisf-=0.1;
      randommove();
    }
    id_d=KEBA.closest(this,KEBA.RAIN);
    if (id_d[1]<=rp){
      satisf-=0.25*thirst;
    }
    id_d=KEBA.closest(this,KEBA.ROCK);
    if (id_d[1]<=radius){
      satisf-=0.5;
    }
    id_d=KEBA.closest(this,KEBA.LIGHTNING);
    if (id_d[1]<=radius){
      satisf-=0.6;
    }
    return satisf;
  }
  public float approachRain(){
    float satisf=0;
    float id_d[]=KEBA.closest(this,KEBA.RAIN);
    if ((id_d[1]>0)&&(id_d[1]<=rp)){
      satisf+=0.25*thirst;
      approach(KEBA.RAIN);
    } else{
      satisf-=0.1;
      randommove();
    }
    id_d=KEBA.closest(this,KEBA.FOOD);
    if (id_d[1]<=rp){
      satisf-=0.2*hunger;
    }
    id_d=KEBA.closest(this,KEBA.ROCK);
    if (id_d[1]<=radius){
      satisf-=0.5;
    }
    id_d=KEBA.closest(this,KEBA.LIGHTNING);
    if (id_d[1]<=radius){
      satisf-=0.6;
    }
    return satisf;
  }
  public float approachLightning(){
    float satisf=0;
    float id_d[]=KEBA.closest(this,KEBA.LIGHTNING);
    if ((id_d[1]>0)&&(id_d[1]<=rp)){
      satisf+=0.01;
      approach(KEBA.LIGHTNING);
    } else{
      satisf-=0.01;
      randommove();
    }
    id_d=KEBA.closest(this,KEBA.RAIN);
    if (id_d[1]<=rp){
      satisf-=0.4*thirst;
    }
    id_d=KEBA.closest(this,KEBA.FOOD);
    if (id_d[1]<=rp){
      satisf-=0.4*hunger;
    }
    id_d=KEBA.closest(this,KEBA.ROCK);
    if (id_d[1]<=radius){
      satisf-=0.5;
    }
    id_d=KEBA.closest(this,KEBA.LIGHTNING);
    if (id_d[1]<=radius){
      satisf-=0.6;
    }

    return satisf;
  }
  public float avoid(){
    float satisf=0;
    //th+=Math.PI/2+KEBA.rnd.nextFloat()*Math.PI/*/10-Math.PI/20*/;
    if (KEBA.rnd.nextBoolean())
      th+=Math.PI/2+KEBA.rnd.nextFloat()*Math.PI/10-Math.PI/20;//turn left ~90 deg...
    else
      th-=Math.PI/2+KEBA.rnd.nextFloat()*Math.PI/10-Math.PI/20;//turn right ~90 deg...

    z+=Math.cos(th)*2*(this.radius+KEBA.rnd.nextFloat());//step accordingly to your size...
    x+=Math.sin(th)*2*(this.radius+KEBA.rnd.nextFloat());
    float id_d[]=KEBA.closest(this,KEBA.ROCK);
    if (id_d[1]<=radius){
      satisf+=0.4;
    }
    id_d=KEBA.closest(this,KEBA.LIGHTNING);
    if (id_d[1]<=radius){
      satisf+=0.5;
    }
    satisf-=0.1;
    return satisf;
  }
  public void approach(int type){
    float angle;
    float id_d[]=KEBA.closest(this,type);
    Phenomenon ph=KEBA.getPhenomenon((long)id_d[0]);
    if (z<ph.z)
      th=(float)Math.atan((ph.x-x)/(ph.z-z));//turn towards the phenomena
    else
      th=(float)(Math.atan((ph.x-x)/(ph.z-z))+Math.PI);//turn towards the phenomena
    z+=Math.cos(th)*this.radius;//just follow the angle
    x+=Math.sin(th)*this.radius;//just follow the angle
  }

  public void randommove(){
    th+=(KEBA.rnd.nextFloat()*Math.PI/4)-Math.PI/8;//get random angle
    z+=Math.cos(th)*this.radius;//just follow the angle
    x+=Math.sin(th)*this.radius;//just follow the angle
  }

}