
/**
 * Title:        "Knowledge Emerging from Behaviour" Architecture<p>
 * Description:  <p>
 * Copyright:    Copyright (c) Carlos Gershenson, C.Gershenson@sussex.ac.uk<p>
 * Company:      COGS, University of Sussex<p>
 * @author Carlos Gershenson, C.Gershenson@sussex.ac.uk
 * @version 3.0
 */
package keba;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.border.*;
import javax.vecmath.*;
import keba.tools.Tools;

public class AnimatKEBAFrame extends AnimatFrame {
  AnimatKEBA a;
  JComboBox behaviourCB;
  JLabel KEBL;
  JComboBox KEBCB;
  JTextField KEBTF;
  JButton KEBB;
  JToggleButton dokebTB;
  JPanel panel;
  FlowLayout flowLayout6;

  public AnimatKEBAFrame(Animat na) {
    super(na);
  }

  void jbInit(Animat na) throws Exception {
    a=(AnimatKEBA)na;
    as=(Animat)a;
    behaviourCB = new JComboBox();
    KEBL = new JLabel();
    KEBCB = new JComboBox();
    KEBTF = new JTextField();
    KEBB = new JButton();
    dokebTB = new JToggleButton();
    panel = new JPanel();
    flowLayout6 = new FlowLayout();

    jbInitAll();

    this.setBounds(60,50,730,500);

    output = new TextArea(21, 116);
    output.setFont(new java.awt.Font("Monospaced", 0, 10));

    KEBL.setFont(new java.awt.Font("Dialog", 1, 10));
    KEBL.setText("KEB:");
    KEBCB.setFont(new java.awt.Font("Dialog", 1, 10));
    KEBCB.setToolTipText("Select KEB parameter");
    KEBCB.addItem("MaxLevels");
    KEBCB.addItem("MaxKoncepts");
    KEBCB.addItem("MaxPersistence");
    KEBCB.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        KEBCB_actionPerformed(e);
      }
    });
    KEBTF.setFont(new java.awt.Font("Dialog", 0, 10));
    KEBTF.setToolTipText("KEB parameters");
    KEBTF.setText(Tools.str(a.maxlevs));
    KEBTF.setPreferredSize(new Dimension(30, 19));
    KEBB.setFont(new java.awt.Font("Dialog", 1, 10));
    KEBB.setToolTipText("Set KEB parameter");
    KEBB.setMargin(new Insets(0, 4, 0, 4));
    KEBB.setText("Set");
    KEBB.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        KEBB_actionPerformed(e);
      }
    });
    behaviourCB.setFont(new java.awt.Font("Dialog", 1, 10));
    behaviourCB.setToolTipText("Select behaviour of animat");
    behaviourCB.addItem("Wander");
    behaviourCB.addItem("Circle");
    behaviourCB.addItem("Linear");
    behaviourCB.addItem("Static");
    behaviourCB.setSelectedIndex(a.behaviour);
    behaviourCB.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        behaviourCB_actionPerformed(e);
      }
    });

    dokebTB.setMargin(new Insets(0, 4, 0, 4));
    dokebTB.setToolTipText("Abstract koncepts from perceptions");
    dokebTB.setText("KEB");
    dokebTB.setActionCommand("KEB");
    dokebTB.setFont(new java.awt.Font("Dialog", 1, 10));
    dokebTB.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dokebTB_actionPerformed(e);
      }
    });
    panel.setPreferredSize(new Dimension(450, 220));
    panel.setLayout(flowLayout6);

    this.getContentPane().add(output, null);
    this.getContentPane().add(panel, null);
    panel.add(nameL, null);
    panel.add(nameTF, null);
    panel.add(nameB, null);
    panel.add(clearB, null);
    panel.add(verboseTB, null);
    panel.add(immortalTB, null);
    panel.add(trailTB, null);
    panel.add(rmTrailB, null);
    panel.add(dokebTB, null);
    panel.add(jPanel1, null);
    panel.add(new JLabel("   "), null);
    panel.add(internalP, null);
    panel.add(KEBL, null);
    panel.add(KEBCB, null);
    panel.add(KEBTF, null);
    panel.add(KEBB, null);
    //panel.add(behaviourCB, null);
    panel.add(removeB, null);
    this.getContentPane().add(new JLabel("                                   "), null);
    this.getContentPane().add(new JLabel("                                   "), null);
    this.getContentPane().add(new JLabel("     "), null);

    if (a.dokeb){
      dokebTB.setSelected(true);
    }

    this.show();
    this.toBack();
  }

  public void step(){
    stepState();
  }

  void KEBB_actionPerformed(ActionEvent e) {
    boolean ok=true;
    float prov = 0;
    try {
      prov = Float.valueOf(KEBTF.getText()).floatValue();
    }
    catch (NumberFormatException nfe) {
      output.append("Please fill the KEB field correctly!\n");
      ok = false;
    }
    if (ok) {
      switch(KEBCB.getSelectedIndex()){
        case 0://maxlevs
          int provi=(int)Math.floor(prov);
          if (provi<0){
            output.append("MaxLevels cannot be negative!");
          }else{
            a.maxlevs=provi;
            a.keb.trimlev(a.maxlevs);
            if (a.verbose)
              output.append("New MaxLevels: "+Tools.str(a.maxlevs)+"\n");
          }
        break;
        case 1://maxkons
          provi=(int)Math.floor(prov);
          if (provi<0){
            output.append("MaxKoncepts cannot be negative!");
          }else if (provi<a.kons){
            output.append("MaxKoncepts cannot lesser than the actual koncepts");
            a.maxkons=a.kons;
            output.append("New MaxKoncepts: "+Tools.str(a.maxkons)+"\n");
          }else{
            a.maxkons=provi;
            if (a.verbose)
              output.append("New MaxKoncepts: "+Tools.str(a.maxkons)+"\n");
          }
        break;
        case 2://maxpersist
          provi=(int)Math.floor(prov);
          if (provi<0){
            output.append("MaxPersist cannot be negative!");
          }else{
            a.maxpersist=provi;
            if (a.verbose)
              output.append("New MaxPersist (maximum persistance of unused koncepts): "
                            +Tools.str(a.maxpersist)+"\n");
          }
        break;


      }
    }
  }

  void KEBCB_actionPerformed(ActionEvent e) {
    switch(KEBCB.getSelectedIndex()){
      case 0:
        KEBTF.setText(Tools.str(a.maxlevs));
      break;
      case 1:
        KEBTF.setText(Tools.str(a.maxkons));
      break;
      case 2:
        KEBTF.setText(Tools.str(a.maxpersist));
      break;
    }
  }

  void behaviourCB_actionPerformed(ActionEvent e) {
    a.behaviour= behaviourCB.getSelectedIndex();
    if (a.verbose){
      switch(a.behaviour){
        case 0:
          output.append("Wandering behaviour set...\n");
        break;
        case 1:
          output.append("Circular behaviour set...\n");
        break;
        case 2:
          output.append("Linear behaviour set...\n");
        break;
        case 3:
          output.append("Static behaviour set...\n");
        break;
        default:
          output.append("Invalid behaviour!\n");
        break;
      }
    }
  }

  void dokebTB_actionPerformed(ActionEvent e) {
    if(a.dokeb)
      output.append("Stop perceiving and using KEBA... actions will be random\n");
    else
      output.append("Use KEBA!\n");
    a.dokeb =! a.dokeb;
  }
  void noiseB_actionPerformed(ActionEvent e) {
    boolean ok=true;
    float nprov = 0;
    try {
      nprov = Float.valueOf(noiseTF.getText()).floatValue();
    }
    catch (NumberFormatException nfe) {
      output.append("Please fill the Noise field correctly!\n");
      ok = false;
    }
    if (ok) {
      a.noise=nprov;
      a.keb.updateNoise(nprov);
      if (a.verbose)
        output.append("Noise for perception set at: "+Tools.str(a.noise)+"\n");
    }
  }

}