
/**
 * Title:        "Knowledge Emerging from Behaviour" Archistecture<p>
 * Description:  <p>
 * Copyright:    Copyright (c) Carlos Gershenson, C.Gershenson@sussex.ac.uk<p>
 * Company:      COGS, University of Sussex<p>
 * @author Carlos Gershenson, C.Gershenson@sussex.ac.uk
 * @version 3.0
 */
package keba;

import java.awt.Color;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.*;
import keba.tools.*;

public class AnimatMaes extends Animat {
  int action=-1;//equivalent to behaviours in Maes model...
  BehaviourMaes[] behs;//network of behaviours (agents)
  float safety=4.0f;//"motivation" for avoiding rocks...
  float curiosity=1.2f;//"motivation" for exploring...
  float boredom=1.5f;//"motivation" for doing nothing...
  float theta=1.0f;//threshold
  float phi=0.5f;//amount of proposition injecting
  float gamma=0.5f;//amount of goal injecting
  float pred=0.2f;//precedessor strength
  float succ=0.2f;//successor strenght
  float conf=0.2f;//conflicter strenght
  float maxAct=5.0f;//maximum activation of behaviours
  float decayRate=0.33f;//each step behaviours multiplied by decay for t+1...
  //float delta;//amount of energy a goal takes away from the network
  //actions
  public static final int NACT=8;//number of actions
  public static final int NONE=0;
  public static final int EXPLORE=1;
  public static final int EAT=2;
  public static final int DRINK=3;
  public static final int APPROACH_FOOD=4;
  public static final int APPROACH_RAIN=5;
  public static final int APPROACH_LIGHTNING=6;
  public static final int AVOID=7;
  //links
  public static final int SUCCESSOR=-1;
  public static final int PREDECESSOR=1;
  public static final int CONFLICTER=2;

  public AnimatMaes(float nz, float nx, float nradius, long nid) {
    super(nz,nx,nradius,new Color(0.1f, 1.0f, 0.1f), nid,KEBA.ANIMAT_MAES);
    init3D();

    behs=new BehaviourMaes[NACT];
    for (int i=0;i<NACT;i++){
      behs[i]=new BehaviourMaes(i);
    }
    behs[NONE].links[EXPLORE]=SUCCESSOR;//EXPLORE successor of NONE
    behs[EXPLORE].links[NONE]=PREDECESSOR;//EXPLORE predecessor of NONE
    behs[EXPLORE].links[APPROACH_FOOD]=SUCCESSOR;//EXPLORE successor of APPROACH_...
    behs[EXPLORE].links[APPROACH_RAIN]=SUCCESSOR;
    behs[EXPLORE].links[APPROACH_LIGHTNING]=SUCCESSOR;
    behs[EXPLORE].links[AVOID]=PREDECESSOR;
    behs[AVOID].links[EXPLORE]=SUCCESSOR;
    behs[APPROACH_FOOD].links[EXPLORE]=PREDECESSOR;
    behs[APPROACH_RAIN].links[EXPLORE]=PREDECESSOR;
    behs[APPROACH_LIGHTNING].links[EXPLORE]=PREDECESSOR;
    behs[APPROACH_LIGHTNING].links[APPROACH_RAIN]=SUCCESSOR;
    behs[APPROACH_RAIN].links[APPROACH_LIGHTNING]=PREDECESSOR;
    behs[APPROACH_RAIN].links[APPROACH_FOOD]=SUCCESSOR;
    behs[APPROACH_FOOD].links[APPROACH_RAIN]=PREDECESSOR;
    behs[APPROACH_FOOD].links[EAT]=SUCCESSOR;
    behs[EAT].links[APPROACH_FOOD]=PREDECESSOR;
    behs[APPROACH_RAIN].links[DRINK]=SUCCESSOR;
    behs[DRINK].links[APPROACH_RAIN]=PREDECESSOR;
    //create AnimatFrame
    af=new AnimatMaesFrame(this);
  }

  public static String actType(int type){
    if (type==NONE) return new String("No Action");
    if (type==EXPLORE) return new String("Exploring");
    if (type==EAT) return new String("Eating");
    if (type==DRINK) return new String("Drinking");
    if (type==APPROACH_FOOD) return new String("Approaching Food");
    if (type==APPROACH_RAIN) return new String("Approaching Rain");
    if (type==APPROACH_LIGHTNING) return new String("Approaching Lightning");
    if (type==AVOID) return new String("Avoiding");
    return new String("Unknown action...");
  }

  void restore(){
    af=new AnimatMaesFrame(this);
  }

  void step() {
    if (!dead){
      msg="+ Animat "+name+" at: "+Tools.str(z)+", "+Tools.str(x)+
          ", angle: "+Tools.str(th)+", t: "+Tools.str(time)+"\n";
      behave();
      for (int i=0;i<NACT;i++){
        msg+=behs[i].show();
      }
      msg+=actType(action)+"\n";
    }
    stepState();
  }

  public void behave(){//ASM...
    for (int i=0;i<NACT;i++){
      behs[i].exec=false;
      behs[i].a*=decayRate;
      if(behs[i].a>maxAct)
        behs[i].a=maxAct;
    }
    behs[DRINK].a+=phi*rainAtRange();
    behs[DRINK].a+=gamma*thirst;
    if (rainAtRange()>0) behs[DRINK].exec=true;
    behs[EAT].a+=phi*foodAtRange();
    behs[EAT].a+=gamma*hunger;
    if (foodAtRange()>0) behs[EAT].exec=true;
    behs[APPROACH_RAIN].a+=phi*rainPerc();
    behs[APPROACH_RAIN].a+=gamma*thirst;
    if (rainPerc()>0) behs[APPROACH_RAIN].exec=true;
    behs[APPROACH_FOOD].a+=phi*foodPerc();
    behs[APPROACH_FOOD].a+=gamma*hunger;
    if (foodPerc()>0) behs[APPROACH_FOOD].exec=true;
    if (lightningPerc()>0) behs[APPROACH_LIGHTNING].exec=true;
    behs[AVOID].a+=phi*(rockNear()+lightningNear());
    behs[AVOID].a+=gamma*safety;
    if (rockNear()>0) behs[AVOID].exec=true;
    behs[EXPLORE].a+=gamma*(hunger+thirst+curiosity)/3;
    behs[EXPLORE].exec=true;
    behs[NONE].a+=gamma*(boredom-(hunger+thirst))/2;
    behs[NONE].exec=true;

    for (int i=0;i<NACT;i++){
      for (int j=0;j<NACT;j++){
        if ((behs[i].links[j]==SUCCESSOR)&&(behs[i].exec))
          behs[j].a+=pred*behs[i].a;
        if ((behs[i].links[j]==PREDECESSOR)&&(!behs[i].exec))
          behs[j].a+=succ*behs[i].a;
        if (behs[i].links[j]==CONFLICTER)
          behs[j].a-=conf*behs[i].a;
      }
    }

    boolean winner=false;
    float maxA=-1;//max activation
    float th=theta;
    while (!winner){
      for (int i=0;i<NACT;i++){
        if ((behs[i].exec)&&(behs[i].a>th)&&(behs[i].a>maxA)){
          action=i;
          maxA=behs[i].a;
        }
      }
      if (maxA>-1){
        winner=true;
        behs[action].a=0;
      }else
        th*=0.9;
    }
    act(action);
  }

  public void act(int type){
    if (type==NONE) noAct();
    if (type==EXPLORE) explore();
    if (type==EAT) eat();
    if (type==DRINK) drink();
    if (type==APPROACH_FOOD) approachFood();
    if (type==APPROACH_RAIN) approachRain();
    if (type==APPROACH_LIGHTNING) approachLightning();
    if (type==AVOID) avoid();
  }

//perceptual functions (megacheat!)
  public int foodPerc(){
    float id_d[]=KEBA.closest(this,KEBA.FOOD);
    if (id_d[1]<=rp){
      return 1;
    }
    return 0;
  }
  public int rainPerc(){
    float id_d[]=KEBA.closest(this,KEBA.RAIN);
    if (id_d[1]<=rp){
      return 1;
    }
    return 0;
  }
  public int lightningPerc(){
    float id_d[]=KEBA.closest(this,KEBA.LIGHTNING);
    if (id_d[1]<=rp){
      return 1;
    }
    return 0;
  }
  public int foodAtRange(){
    float id_d[]=KEBA.closest(this,KEBA.FOOD);
    if (id_d[1]<=0){
      return 1;
    }
    return 0;
  }
  public int rainAtRange(){
    Phenomenon ph;
    float id_d[]=KEBA.closest(this,KEBA.RAIN);
    if (id_d[1]<=0){
      return 1;
    }
    return 0;
  }
  public int lightningNear(){
    float thr;
    float id_d[]=KEBA.closest(this,KEBA.LIGHTNING);
    if (id_d[1]<=radius){
      return 1;
    }
    return 0;
  }
  public int rockNear(){
    float thr;
    float id_d[]=KEBA.closest(this,KEBA.ROCK);
    if (id_d[1]<=radius){
      Phenomenon ph=KEBA.getPhenomenon((long)id_d[0]);
      if (z<ph.z)
        thr=(float)Math.atan((ph.x-x)/(ph.z-z));//angle towards rock
      else
        thr=(float)(Math.atan((ph.x-x)/(ph.z-z))+Math.PI);//angle towards rock
      if (thr<0) thr+=2*(float)Math.PI;
      if (thr!=0) thr=thr%(2*(float)Math.PI);
      if (Math.abs(th-thr)<=(Math.PI/4)){
        return 1;
      }
    }
    return 0;
  }

//actions
  public void noAct(){
    action=NONE;
  }
  public void explore(){
    th+=(KEBA.rnd.nextFloat()*Math.PI/4)-Math.PI/8;//get random angle
    z+=Math.cos(th)*this.radius;//just follow the angle
    x+=Math.sin(th)*this.radius;//just follow the angle
    action=EXPLORE;
  }
  public void eat(){
    Phenomenon ph, nph;
    float id_d[]=KEBA.closest(this,KEBA.FOOD);
    ph=KEBA.getPhenomenon((long)id_d[0]);
    if (ph.radius>0.1){
      nph=new Food(ph.z,ph.x,ph.radius-0.1f,ph.id);
      KEBA.addPhenomenon(nph, false);
    }
    KEBA.removePhenomenon(ph, false);
    hunger-=0.1;
    action=EAT;
  }
  public void drink(){
    thirst-=0.1;
    action=DRINK;
  }
  public void approachFood(){
    approach(KEBA.FOOD);
    action=APPROACH_FOOD;
  }
  public void approachRain(){
    approach(KEBA.RAIN);
    action=APPROACH_RAIN;
  }
  public void approachLightning(){
    approach(KEBA.LIGHTNING);
    action=APPROACH_LIGHTNING;
  }
  public void avoid(){
    th+=Math.PI/2+KEBA.rnd.nextFloat()*Math.PI/10-Math.PI/20;//turn left ~90 deg...
    z+=Math.cos(th)*2*(this.radius+KEBA.rnd.nextFloat());//step accordingly to your size...
    x+=Math.sin(th)*2*(this.radius+KEBA.rnd.nextFloat());
    action=AVOID;
  }
  public void approach(int type){
    float angle;
    float id_d[]=KEBA.closest(this,type);
    Phenomenon ph=KEBA.getPhenomenon((long)id_d[0]);
    if (z<ph.z)
      th=(float)Math.atan((ph.x-x)/(ph.z-z));//turn towards the phenomena
    else
      th=(float)(Math.atan((ph.x-x)/(ph.z-z))+Math.PI);//turn towards the phenomena
    z+=Math.cos(th)*this.radius;//just follow the angle
    x+=Math.sin(th)*this.radius;//just follow the angle
  }

}