
/**
 * Title:        "Knowledge Emerging from Behaviour" Archistecture<p>
 * Description:  <p>
 * Copyright:    Copyright (c) Carlos Gershenson, C.Gershenson@sussex.ac.uk<p>
 * Company:      COGS, University of Sussex<p>
 * @author Carlos Gershenson, C.Gershenson@sussex.ac.uk
 * @version 3.0
 */
package keba;

import java.awt.Color;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.*;
import keba.tools.*;

public class AnimatNN extends Animat {
  float [] inputs;
  float [] hidden;
  float [] outputs;
  float [][] w;//weights
  float [][] v;//weights
  float thr=1.01f;//threshold

  float noise=0.5f;//pref. between 0 and 1... but of course... if you're perverse enought... try it!

  //directions
  public static final int LEFT=-1;
  public static final int RIGHT=1;

  //inputs
  public static final int NINPUTS=8;
  public static final int WL=0;//water left (rain)
  public static final int WR=1;//water right
  public static final int FL=2;//food...
  public static final int FR=3;
  public static final int RL=4;//rock...
  public static final int RR=5;
  public static final int THIRST=6;
  public static final int HUNGER=7;
  //hidden
  public static final int NHIDDEN=6;
  //outputs
  public static final int NOUTPUTS=2;
  public static final int LM=0;//left motor
  public static final int RM=1;//right motor

  public AnimatNN(float nz, float nx, float nradius, long nid) {
    super(nz,nx,nradius,new Color(0.73f, 0.05f, 1.0f), nid,KEBA.ANIMAT_NN);
    init3D();
    //create AnimatFrame
    af=new AnimatNNFrame(this);

    //hand engineered NN...
    w=new float [NINPUTS][NHIDDEN];
    v=new float [NHIDDEN][NOUTPUTS];
    w[WL][WL]=1;
    w[WR][WR]=1;
    w[FL][FL]=1;
    w[FR][FR]=1;
    w[RL][RL]=4;
    w[RR][RR]=4;
    w[THIRST][WL]=2;
    w[THIRST][WR]=2;
    w[HUNGER][FL]=2;
    w[HUNGER][FR]=2;
    v[WL][RM]=1;
    v[WR][LM]=1;
    v[FL][RM]=1;
    v[FR][LM]=1;
    v[RL][LM]=4;
    v[RR][RM]=4;
  }

  void restore(){
    af=new AnimatNNFrame(this);
  }

  void step() {
    if (!dead){
      msg="+ Animat "+name+" at: "+Tools.str(z)+", "+Tools.str(x)+
          ", angle: "+Tools.str(th)+", t: "+Tools.str(time)+"\n";
      move();
    }
    stepState();
  }

  public void move(){//ASM???
    int perc;
    float sum;
    inputs=new float[NINPUTS];
    hidden=new float[NHIDDEN];
    outputs=new float[NOUTPUTS];

    perc=rainPerc();
    if (perc==LEFT){
      inputs[WL]=1;
    } else if (perc==RIGHT){
      inputs[WR]=1;
    }
    perc=foodPerc();
    if (perc==LEFT){
      inputs[FL]=1;
    } else if (perc==RIGHT){
      inputs[FR]=1;
    }
    perc=rockNear();
    if (perc==LEFT){
      inputs[RL]=1;
    } else if (perc==RIGHT){
      inputs[RR]=1;
    }
    inputs[THIRST]=thirst;
    inputs[HUNGER]=hunger;

    for (int i=0;i<NHIDDEN;i++){
      sum=0;
      for (int j=0;j<NINPUTS;j++){
        sum+=inputs[j]*w[j][i];
      }
      if (sum>thr)
        hidden[i]=sum;
      else
        hidden[i]=0;
    }
    for (int i=0;i<NOUTPUTS;i++){
      sum=0;
      for (int j=0;j<NHIDDEN;j++){
        sum+=hidden[j]*v[j][i];
      }
      outputs[i]=sum;//thershold=0
    }

    msg+="Inputs: "+Tools.showVec(inputs);
    msg+="Hidden: "+Tools.showVec(hidden);
    msg+="Outputs: "+Tools.showVec(outputs);

    //noise...
    rightMotor(noise*(Math.PI/20)*energy*(hunger+thirst)*KEBA.rnd.nextFloat());
    leftMotor(noise*(Math.PI/20)*energy*(hunger+thirst)*KEBA.rnd.nextFloat());

    if ((outputs[RM]>0)||(outputs[LM]>0)){
      if (outputs[RM]>outputs[LM]){
        rightMotor((Math.PI/4)*energy);
      } else
      if (outputs[RM]<outputs[LM]){
        leftMotor((Math.PI/4)*energy);
      } else {//equal...
        rightMotor((Math.PI/4)*energy);
        leftMotor((Math.PI/4)*energy);
      }

    }

//    rightMotor((Math.PI/4)*energy*outputs[RM]);
//    leftMotor((Math.PI/4)*energy*outputs[LM]);

    if ((foodAtRange())&&(hunger>0.0)){
      eat();
    }
    if ((rainAtRange())&&(thirst>0.0)){
      drink();
    }
  }

//perceptual functions
  public int LRPh(Phenomenon ph){//checks if ph is left or right of animat...
    if ( ( ((th<(Math.PI/2))||(th>=(3*Math.PI/2)))&&( ph.x>=x+(Math.tan(th)*(ph.z-z)) ) ) ||
         ( ((th>=(Math.PI/2))&&(th<(3*Math.PI/2)))&&( ph.x<x+(Math.tan(th)*(ph.z-z)) ) ) ){
      return LEFT;
    }else{
      return RIGHT;
    }
  }

  public int foodPerc(){
    float id_d[]=KEBA.maxpond(this,KEBA.FOOD);
    if (id_d[1]<=rp){
      Phenomenon ph=KEBA.getPhenomenon((long)id_d[0]);
      return LRPh(ph);
    }
    return 0;//none
  }
  public int rainPerc(){
    float id_d[]=KEBA.maxpond(this,KEBA.RAIN);
    if (id_d[1]<=rp){
      Phenomenon ph=KEBA.getPhenomenon((long)id_d[0]);
      return LRPh(ph);
    }
    return 0;//none
  }
  public int rockNear(){
    float id_d[]=KEBA.maxpond(this,KEBA.ROCK);
    if (id_d[1]<=(2*radius)){
      Phenomenon ph=KEBA.getPhenomenon((long)id_d[0]);
      return LRPh(ph);
    }
    return 0;//none
  }
  public boolean foodAtRange(){
    float id_d[]=KEBA.closest(this,KEBA.FOOD);
    if (id_d[1]<=(radius/2)){
      return true;
    }
    return false;
  }
  public boolean rainAtRange(){
    Phenomenon ph;
    float id_d[]=KEBA.closest(this,KEBA.RAIN);
    if (id_d[1]<=0){
      return true;
    }
    return false;
  }


//motors
  public void rightMotor(double thr) {//from ASIA... turn left
    float h = z- (float) Math.cos(th);
    float k = x- (float) Math.sin(th);
    z = (float) h+(float) Math.cos(th+thr);
    x = (float) k+(float) Math.sin(th+thr);
    th += thr;
  }
  public void leftMotor(double thr) {//from ASIA... turn right
    float h = z- (float) Math.cos(th+(Math.PI/2));
    float k = x- (float) Math.sin(th+(Math.PI/2));
    z = (float) h+(float) Math.cos(th+(Math.PI/2)-thr);
    x = (float) k+(float) Math.sin(th+(Math.PI/2)-thr);
    th -= thr;
  }

  public void eat(){
    Phenomenon ph, nph;
    float id_d[]=KEBA.closest(this,KEBA.FOOD);
    ph=KEBA.getPhenomenon((long)id_d[0]);
    if (ph.radius>0.1){
      nph=new Food(ph.z,ph.x,ph.radius-0.1f,ph.id);
      KEBA.addPhenomenon(nph, false);
    }
    KEBA.removePhenomenon(ph, false);
    hunger-=0.1;
  }
  public void drink(){
    thirst-=0.1;
  }
}