
/**
 * Title:        "Knowledge Emerging from Behaviour" Archistecture<p>
 * Description:  <p>
 * Copyright:    Copyright (c) Carlos Gershenson, C.Gershenson@sussex.ac.uk<p>
 * Company:      COGS, University of Sussex<p>
 * @author Carlos Gershenson, C.Gershenson@sussex.ac.uk
 * @version 3.0
 */
package keba;

import java.awt.Color;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.*;
import keba.tools.*;

public class AnimatRule extends Animat {
  int action=-1;
  //actions
  public static final int NACT=8;//number of actions
  public static final int NONE=0;
  public static final int EXPLORE=1;
  public static final int EAT=2;
  public static final int DRINK=3;
  public static final int APPROACH_FOOD=4;
  public static final int APPROACH_RAIN=5;
  public static final int APPROACH_LIGHTNING=6;
  public static final int AVOID=7;

  public AnimatRule(float nz, float nx, float nradius, long nid) {
    super(nz,nx,nradius,new Color(0.96f, 0.1f, 0.02f), nid,KEBA.ANIMAT_RULE);
    init3D();
    //create AnimatFrame
    af=new AnimatRuleFrame(this);
  }

  public static String actType(int type){
    if (type==NONE) return new String("No Action");
    if (type==EXPLORE) return new String("Exploring");
    if (type==EAT) return new String("Eating");
    if (type==DRINK) return new String("Drinking");
    if (type==APPROACH_FOOD) return new String("Approaching Food");
    if (type==APPROACH_RAIN) return new String("Approaching Rain");
    if (type==APPROACH_LIGHTNING) return new String("Approaching Lightning");
    if (type==AVOID) return new String("Avoiding");
    return new String("Unknown action...");
  }

  void restore(){
    af=new AnimatRuleFrame(this);
  }

  void step() {
    if (!dead){
      msg="+ Animat "+name+" at: "+Tools.str(z)+", "+Tools.str(x)+
          ", angle: "+Tools.str(th)+", t: "+Tools.str(time)+"\n";
      rules();
      msg+=actType(action)+"\n";
    }
    stepState();
  }

  public void rules(){//"ASM"...
    if (rockNear()||lightningNear()){
      avoid();
    }else if ((thirst>=0.1)&&(rainPerc())){
      if (rainAtRange()){
        drink();
      }else{
        approachRain();
      }
    }else if ((hunger>=0.1)&&(foodPerc())){
      if (foodAtRange()){
        eat();
      }else{
        approachFood();
      }
    }else if ((thirst>=0.1)&&(lightningPerc())){
      approachLightning();//because it will turn into rain...
    }else if ((hunger>=0.1)&&(rainPerc())){
      approachRain();//because it will turn into food...
    }else if ((thirst>=0.1)||(hunger>=0.1)){
      explore();
    }else{
      noAct();
    }
  }

//perceptual functions (megacheat!)
  public boolean foodPerc(){
    float id_d[]=KEBA.closest(this,KEBA.FOOD);
    if (id_d[1]<=rp){
      return true;
    }
    return false;
  }
  public boolean rainPerc(){
    float id_d[]=KEBA.closest(this,KEBA.RAIN);
    if (id_d[1]<=rp){
      return true;
    }
    return false;
  }
  public boolean lightningPerc(){
    float id_d[]=KEBA.closest(this,KEBA.LIGHTNING);
    if (id_d[1]<=rp){
      return true;
    }
    return false;
  }
  public boolean foodAtRange(){
    float id_d[]=KEBA.closest(this,KEBA.FOOD);
    if (id_d[1]<=0){
      return true;
    }
    return false;
  }
  public boolean rainAtRange(){
    Phenomenon ph;
    float id_d[]=KEBA.closest(this,KEBA.RAIN);
    if (id_d[1]<=0){
      return true;
    }
    return false;
  }
  public boolean lightningNear(){
    float thr;
    float id_d[]=KEBA.closest(this,KEBA.LIGHTNING);
    if (id_d[1]<=radius){
      return true;
    }
    return false;
  }
  public boolean rockNear(){
    float thr;
    float id_d[]=KEBA.closest(this,KEBA.ROCK);
    if (id_d[1]<=radius){
      Phenomenon ph=KEBA.getPhenomenon((long)id_d[0]);
      if (z<ph.z)
        thr=(float)Math.atan((ph.x-x)/(ph.z-z));//angle towards rock
      else
        thr=(float)(Math.atan((ph.x-x)/(ph.z-z))+Math.PI);//angle towards rock
      if (thr<0) thr+=2*(float)Math.PI;
      if (thr!=0) thr=thr%(2*(float)Math.PI);
      if (Math.abs(th-thr)<=(Math.PI/4))
        return true;
    }
    return false;
  }

//actions
  public void noAct(){
    action=NONE;
  }
  public void explore(){
    th+=(KEBA.rnd.nextFloat()*Math.PI/4)-Math.PI/8;//get random angle
    z+=Math.cos(th)*this.radius;//just follow the angle
    x+=Math.sin(th)*this.radius;//just follow the angle
    action=EXPLORE;
  }
  public void eat(){
    Phenomenon ph, nph;
    float id_d[]=KEBA.closest(this,KEBA.FOOD);
    ph=KEBA.getPhenomenon((long)id_d[0]);
    if (ph.radius>0.1){
      nph=new Food(ph.z,ph.x,ph.radius-0.1f,ph.id);
      KEBA.addPhenomenon(nph, false);
    }
    KEBA.removePhenomenon(ph, false);
    hunger-=0.1;
    action=EAT;
  }
  public void drink(){
    thirst-=0.1;
    action=DRINK;
  }
  public void approachFood(){
    approach(KEBA.FOOD);
    action=APPROACH_FOOD;
  }
  public void approachRain(){
    approach(KEBA.RAIN);
    action=APPROACH_RAIN;
  }
  public void approachLightning(){
    approach(KEBA.LIGHTNING);
    action=APPROACH_LIGHTNING;
  }
  public void avoid(){
    th+=Math.PI/2+KEBA.rnd.nextFloat()*Math.PI/10-Math.PI/20;//turn left ~90 deg...
    z+=Math.cos(th)*2*(this.radius+KEBA.rnd.nextFloat());//step accordingly to your size...
    x+=Math.sin(th)*2*(this.radius+KEBA.rnd.nextFloat());
//    th+=Math.PI/2;//turn left 90 deg...
//    z+=Math.cos(th)*this.radius;//step accordingly to your size...
//    x+=Math.sin(th)*this.radius;
    action=AVOID;
  }
  public void approach(int type){
    float angle;
    float id_d[]=KEBA.closest(this,type);
    Phenomenon ph=KEBA.getPhenomenon((long)id_d[0]);
    if (z<ph.z)
      th=(float)Math.atan((ph.x-x)/(ph.z-z));//turn towards the phenomena
    else
      th=(float)(Math.atan((ph.x-x)/(ph.z-z))+Math.PI);//turn towards the phenomena
    z+=Math.cos(th)*this.radius;//just follow the angle
    x+=Math.sin(th)*this.radius;//just follow the angle
  }

}