
/**
 * Title:        "Knowledge Emerging from Behaviour" Architecture<p>
 * Description:  <p>
 * Copyright:    Copyright (c) Carlos Gershenson, C.Gershenson@sussex.ac.uk<p>
 * Company:      COGS, University of Sussex<p>
 * @author Carlos Gershenson, C.Gershenson@sussex.ac.uk
 * @version 3.0
 */
package keba;

import java.io.Serializable;
import keba.tools.Tools;

public class Concept extends Object implements Serializable, Cloneable{
  String n;//concept "name" (animats don't use names, this is for our reference)
  float v;//value of the concept (representing "actual" value)
  float a;//activation of the concept (representing "historical" values)
  float s;//stability: represents how stable is the activation
  float []acts;//links to actions
  float hyp_max;//maximum link of concept to action (decays as linked action changes)...
  String msg="";
  int level;
  long id;//unique ID
  public static long lastID;//for unique ID for concepts
  public static final float IOTA=2;//importance of previous experiences
                                   //persistance factor (IOTA>=0... or not???)
  public static final float KAPPA=0.08f;//speed of stability (*not even higher than noise)
  public static final float ACTIV_POT=3f;//potence of activation^level
  //for MetaConcepts:
  public static final float R1_INI=0.15f;//initial r1
  public static final float R2_INI=0.45f;//initial r2
  public static final float ETA_CTR=0.05f;//parameter for adjusting centres of concepts
  public static final float ZETA_R=0.01f;//parameter for adjusting radiuses of concepts
  //for Hyperbolic adjustments of links concept-action:
  public static final float HYP_MAX=1.0f;//maximum strenght of link concept-action
  public static final float HYP_MIN=0.0f;//minimum strenght of link concept-action
  public static final float DELTA=10.0f;//length of hyperbolic convergence
  public static final float RHO=0.1f;//speed of hyperbolic convergence
  public static final float XI=0.005f;//limit for adjusting/exploring weights
  public static final float MAX_DECAY=0.1f;//decay of maximum strength

  public Concept(int lev){
    n="";
    v=0;
    a=0;
    s=0;
    acts=new float[AnimatKEBA.NACT];
    for (int i=0;i<AnimatKEBA.NACT;i++){//random init of weights with medium values
      acts[i]=(KEBA.rnd.nextFloat()*0.2f)+0.4f;
    }
    hyp_max=HYP_MAX;
    level=lev;
    id=assignID();
  }
  public Concept(String name, float value,int lev){
    v=value;
    a=value;
    s=0;
    acts=new float[AnimatKEBA.NACT];
    for (int i=0;i<AnimatKEBA.NACT;i++){//random init of weights with medium values
      acts[i]=((HYP_MAX+HYP_MIN)/2)+(KEBA.rnd.nextFloat()*((HYP_MAX-HYP_MIN)/5))-((HYP_MAX-HYP_MIN)/10);
    }
    hyp_max=HYP_MAX;
    level=lev;
    id=assignID();
    n=new String(String.valueOf(id)+":"+name);
  }

  public static long assignID(){//assigns unique ID to Concepts
    lastID++;
    return lastID;
  }
  public static void initID( long l){//need to set max ID while loading sims!
    lastID=l;
  }

  protected Object clone() throws CloneNotSupportedException{
    return super.clone();
  }

  void update(float value){
    float an;//new activation
    if (level==0){
      an=value;
    }else{
      an=(float)((value+(a*IOTA*Math.pow(ACTIV_POT,level)))/
                    ((IOTA*Math.pow(ACTIV_POT,level))+1));//new activation
    }
    s+=(KAPPA/*/Math.pow(ACTIV_POT,level)*/)-Math.abs(a-an);
    if (s>1) s=1;
    if (s<0) s=0;
    v=value;
    a=an;
  }

  String show(int mode){
    String str = new String(n+". ");
    if (mode==KEB.DEL){
      str+="v: "+Tools.str(v)
        +", a: "+Tools.str(a)
        +", s: "+Tools.str(s)
        +",";
    }
    if (mode==KEB.INI){
      str+=msg;
    }
    str+="act:";
    for (int i=0;i<acts.length;i++){
      str+=" "+Tools.str(acts[i]);
    }
    str+=" max:"+Tools.str(hyp_max);
    return str;
  }

  public void inc(int act, float satisf){
    //float maxact=Tools.getMax(acts);
    acts[act]=hyp_max;
    //hyp_max-=MAX_DECAY;
     //decay maximum as concept changes action to be linked to...
     //this avoids "switching" concepts from contributing to actions
     //i.e. concepts which are active when different actions should be taken
     //like hardness with foods and rocks...
    //if (hyp_max<(HYP_MIN+MAX_DECAY)) hyp_max=MAX_DECAY;
    //if (hyp_max<HYP_MIN) hyp_max=HYP_MIN;
  }
  public void dec(int act, float satisf){
    if (acts[act]>((HYP_MAX+HYP_MIN)/2)){
      acts[act]-=MAX_DECAY;
    }else{
      acts[act]=HYP_MIN;
    }
  }
  public void hypinc(int act, float satisf){
    acts[act]=hypinc(acts[act],satisf);
  }
  public void hypdec(int act, float satisf){
    acts[act]=hypdec(acts[act],satisf);
  }
  public float hypinc(float x, float satisf) {//hyperbolic increment
    double x2,pot, xn;
    x2 = (-1/(x-HYP_MIN-(1/-DELTA)))+DELTA+RHO;
    pot = (-1/(x2-DELTA))+HYP_MIN+(1/-DELTA);
    pot=x+Math.abs(satisf);
    if(pot < hyp_max) {
      xn = pot;
    } else {
      xn = hyp_max;
    }
    return (float)xn;
  }
  public float hypdec(float x, float satisf) {//hyperbolic decrement
    double x1,pot, xn;
    x1 = (1/(x-hyp_max-(1/DELTA)))+DELTA+RHO;
    pot = (1/(x1-DELTA))+hyp_max+(1/DELTA);
    pot=x-Math.abs(satisf);
    if(pot > HYP_MIN) {
      xn = pot;
    } else {
      xn = HYP_MIN;
    }
    return (float)xn;
  }

}