
/**
 * Title:        "Knowledge Emerging from Behaviour" Architecture<p>
 * Description:  <p>
 * Copyright:    Copyright (c) Carlos Gershenson, C.Gershenson@sussex.ac.uk<p>
 * Company:      COGS, University of Sussex<p>
 * @author Carlos Gershenson, C.Gershenson@sussex.ac.uk
 * @version 3.0
 */
package keba;

import java.util.Vector;
import keba.tools.Tools;

public class KEB extends Vector {
//Has Concepts or MetaConcepts as elements...
  AnimatKEBA a;//animat which has KEB
  KEB meta;//higher level
  Concept c;
  MetaConcept mc;
  int level, maxlevs, maxkons, maxpersist;
  float noise;
  public static final int INI=0;//initial abstraction (only concepts)
  public static final int DEL=1;//deliberative abstraction (with metaconcepts...)
  public static final int REC=2;//recursive abstraction (with metaconcepts...) (continue)

  public KEB(AnimatKEBA an, int lev, int maxlev, int maxkon, int maxpers, float nois) {
    super(20,5);
    a=an;
    meta=null;
    level=lev;
    maxlevs=maxlev;
    maxkons=maxkon;
    maxpersist=maxpers;
    noise=nois;
  }
  public KEB(KEB keb){
    super(20,5);
    a=keb.a;
    meta=null;
    level=keb.level;
    maxlevs=keb.maxlevs;
    maxkons=keb.maxkons;
    maxpersist=keb.maxpersist;
    noise=keb.noise;
    for (int i=0;i<keb.size();i++){
      c=(Concept)keb.elementAt(i);
      try{
        this.add(c.clone());
      }catch (CloneNotSupportedException cnse){}
    }
  }

  void addKoncept(Concept c){
    addElement(c);
    a.inckons();
  }

  void removeKonceptAt(int index){
    MetaConcept metac=(MetaConcept)this.elementAt(index);
    if (meta!=null){//eliminate children recursively
      for (int i=meta.size()-1;i>=0;i--){
        mc=(MetaConcept)meta.elementAt(i);
        if (mc.isExactParent(metac)){
          meta.removeKonceptAt(i);
        }
      }
    }
    this.removeElementAt(index);
    a.deckons();
  }

  void updateElementAt(float v, int i){
    c=(Concept)this.elementAt(i);
    c.update(v);
    this.setElementAt(c,i);
  }

  public void doAbstraction(long time, int mode){
    if (meta!=null){//update
      if (mode!=DEL){
        //calculate mdl vector and see if there's a similar metaconcept alredy
        boolean newconcept=true;
        for (int i=meta.size()-1;i>=0;i--){
          mc=(MetaConcept)meta.elementAt(i);
          //if (mc.areParents(this)){
          if (mc.exactParents(this)){
            //if yes, update the metaconcept, recalculate the dimension
            mc.update(this, time);
            if (mc.exactParents(this)){
              newconcept=false;
            }
          } else {
            //if not, update with zero value
            if (mode!=INI){
              mc.update(0);
            }
            if ((a.time-mc.t)>a.maxpersist){//forget
              meta.removeKonceptAt(i);
            }
          }
        }
        if (((this.isStable())||(mode==INI))&&(conceptsActive()>1)){
          if ((newconcept) && (a.kons<a.maxkons)){
            meta.addKoncept(new MetaConcept(this,this.level+1));
          }
        }
      }
      if (mode!=INI){
        //recurse abstraction for metaconcepts
        meta.doAbstraction(a.time, REC);
      }
    } else{
      if (((this.isStable())||(mode==INI))&&(conceptsActive()>1)&&(level<maxlevs)){
        if (meta==null){//create next concept level
          meta=new KEB(a,this.level+1, maxlevs, maxkons, maxpersist, noise);
          meta.addKoncept(new MetaConcept(this,this.level+1));
        }
      }
    }
  }

  public void updateZeroesLev1(long time){///change name to updateLev1
    for (int i=0;i<this.size();i++){
      mc=(MetaConcept)elementAt(i);
      if (mc.t!=time){
        mc.update(0);
      }
    }
  }

  float[] getActions(){
    float[] metaact, act=new float[AnimatKEBA.NACT];
    if (meta!=null){
      metaact=meta.getActions();
      for (int i=0;i<act.length;i++){
        act[i]+=metaact[i];
      }
    }
    if (level>0){
      for (int j=0;j<this.size();j++){
        c=(Concept)this.elementAt(j);
        if (c.v>noise){
          for (int i=0;i<act.length;i++){
            //act[i]+=c.v*c.acts[i]/Math.pow(this.level+1,2);
            act[i]+=c.v*c.acts[i]*Math.pow(this.level+1,2);
            //act[i]+=c.v*c.acts[i]*(this.level+1);
            //act[i]+=c.v*c.acts[i];
          }
        }
      }
    }
    return act;
  }

  void linkConcAct(int action, float satisf){//adapts links between concepts and actions
    int act;
    for (int i=0;i<this.size();i++){
      c=(Concept)this.elementAt(i);
      if (c.v/*c.a*/>this.noise){
        act=Tools.getMax(c.acts);
        if (c.acts[act]==Concept.HYP_MIN){//all links are minimum
          //explore new action...
          c.acts[(int)Math.floor(AnimatKEBA.NACT*KEBA.rnd.nextFloat())]=KEBA.rnd.nextFloat()*c.hyp_max;
        }else{
          if (satisf>0){
            if (act==action)
              c.inc(act,satisf);//boost link
            else
              c.hypinc(act,satisf);//slightly increase link
          }
          if (satisf<0){
            if (act==action)
              c.dec(act,satisf);//kill link
            else
              c.hypdec(act,satisf);//slightly decrease link
          }
        }
        this.setElementAt(c,i);
      }
    }
    if (meta!=null){
      meta.linkConcAct(action, satisf);
    }
  }

  boolean isStable(){//true only when all concepts at this level are stable
    for (int i=0;i<this.size();i++){
      c=(Concept)this.elementAt(i);
      if (c.s!=1)
        return false;
    }
    return true;
  }
  int conceptsActive(){//returns how many concepts are active at this level
    int concepts=0;
    for (int i=0;i<this.size();i++){
      c=(Concept)this.elementAt(i);
      if (c.a>this.noise)
        concepts++;
    }
    return concepts;
  }


  void updateNoise(float n){
    this.noise=n;
    if (meta!=null){
      meta.updateNoise(n);
    }
  }

  void trimlev (int maxlev){
    maxlevs=maxlev;
    if (level==maxlevs){
      meta=null;
    }else{
      if (meta!=null)
        meta.trimlev(maxlevs);
    }
  }

  public void writeIniMsg(){
    for (int i=0;i<this.size();i++){
      c=(Concept)this.elementAt(i);
      c.msg+="<v: "+Tools.str(c.v)
        +", a: "+Tools.str(c.a)
        +", s: "+Tools.str(c.s)+"> ";
    }
  }

  public void delIniMsg(){
    for (int i=0;i<this.size();i++){
      c=(Concept)this.elementAt(i);
      c.msg="";
    }
  }

  public String getMessage(){
    String msg="  CONCEPTS AT LEVEL "+String.valueOf(level)+":\n";
    for (int i=0;i<this.size();i++){
      c=(Concept)this.elementAt(i);
      if (level>0)
        msg+=c.show(DEL)+"\n";
      else
        msg+=c.show(INI)+"\n";
    }
    if (meta!=null)//recurse the messages for metaconcepts
      msg+=meta.getMessage();
    return msg;
  }
}