
/**
 * Title:        "Knowledge Emerging from Behaviour" Architecture<p>
 * Description:  <p>
 * Copyright:    Copyright (c) Carlos Gershenson, C.Gershenson@sussex.ac.uk<p>
 * Company:      COGS, University of Sussex<p>
 * @author Carlos Gershenson, C.Gershenson@sussex.ac.uk
 * @version 3.0
 */
package keba;

import keba.tools.Tools;

public class MetaConcept extends Concept {
  KEB parents;//parent concepts...
              //check when KEB grows...
  float r1;//radius of first neighbourhood
  float r2;//radius of second neighbourhood. (r2>=r1)
  long t;//time at which koncept was created

  public MetaConcept(int lev) {
    super(lev);
    r1=R1_INI;
    r2=R2_INI;
    parents=null;
    t=0;
  }
  public MetaConcept(KEB papas, int lev){
    super("",1,lev);
    String n2="";
    r1=R1_INI;
    r2=R2_INI;
    t=papas.a.time;
    parents=new KEB(papas);
    for (int i=parents.size()-1;i>=0;i--){
      Concept c=(Concept)parents.elementAt(i);
      if (c.a<=papas.noise){//trim
        parents.removeElementAt(i);
      }else{//compose name
        n2=String.valueOf(c.id)+"|"+Tools.str(c.a,3)+"-"+n2;
        for (int j=0;j<acts.length;j++){
          acts[j]+=c.acts[j];
        }
      }
    }
    n+=n2;
    for (int j=0;j<acts.length;j++){
      acts[j]=acts[j]/(parents.size()+1);
    }
  }

  boolean areParents(KEB papas){
    for (int i=0;i<parents.size();i++){
      Concept c=(Concept)parents.elementAt(i);
      if (!isParent(c, papas)){
        return false;
      }
    }
    return true;
  }
  boolean isParent(Concept c, KEB papas){
    for (int i=0;i<papas.size();i++){
      Concept c2=(Concept)papas.elementAt(i);
      if ((c.id==c2.id)&&(Math.abs(c.a-c2.a)<this.r2))
        return true;
    }
    return false;
  }

  boolean exactParents(KEB papas){
    for (int i=0;i<papas.size();i++){
      Concept c=(Concept)papas.elementAt(i);
      if (c.a>papas.noise){
        if (!isExactParent(c)){
          return false;
        }
      }else{
        if (isExactParent(c)){
          return false;
        }
      }
    }
    return true;
  }
  boolean isExactParent(Concept c){
    for (int i=0;i<parents.size();i++){
      Concept c2=(Concept)parents.elementAt(i);
      if ((c.id==c2.id)&&(Math.abs(c.a-c2.a)<this.r2))
        return true;
    }
    return false;
  }




  void update(KEB papas, long time){
    //if (t!=time){//not to update more than once a time step...
      t=time;//notice that concept is active
      //calculate v
      float d=0;//distance from center of MDL concept
      Concept c1, c2;
      for (int i=0;i<parents.size();i++){
        c1=(Concept)parents.elementAt(i);
        for (int j=0;j<papas.size();j++){
          c2=(Concept)papas.elementAt(j);
          if (c1.id==c2.id){
            d+=Math.pow(c1.v-c2.v,2);
          }
        }
      }
      d=(float)Math.sqrt(d);
      if (d<=r1)
        v=1;
      else if (d>=r2)
        v=0;
      else
        v=1-((d-r1)/(r2-r1));
      this.update(v);

      //useful only when phenomena have dynamic properties (e.g. change colour, odour, etc...)
      //recalculate center
      //if ((0<v)&&(v<1)){//i.e. only if r1<d<r2
  /*      for (int i=0;i<parents.size();i++){
          c1=(Concept)parents.elementAt(i);
          for (int j=0;j<papas.size();j++){
            c2=(Concept)papas.elementAt(j);
            if (c1.id==c2.id){
              c1.a+=ETA_CTR*v*(c2.a-c1.a);
              c1.v+=ETA_CTR*v*(c2.v-c1.v);
            }
          }
          parents.setElementAt(c1,i);
        }
      //}

      //recalculate radiuses
      if (d>papas.noise){
        if(d<=(r1/2)) r1-=ZETA_R;
        else if (d<=r1) r2-=ZETA_R;
        else if (d<=((r1+r2)/2)) r1+=ZETA_R;
        else if (d<=r2) r2+=ZETA_R;
        if (r2<r1) r2=r1;
      }*/
    //}
  }

  String show(int mode){
    String str = new String(n+". ");
    if (mode==KEB.DEL){
      str+="v: "+Tools.str(v)
        +", a: "+Tools.str(a)
        +", s: "+Tools.str(s)
        +", r1: "+Tools.str(r1)
        +", r2: "+Tools.str(r2)
        +",";
    }
    str+="act:";
    for (int i=0;i<acts.length;i++){
      str+=" "+Tools.str(acts[i]);
    }
    str+=" max:"+Tools.str(hyp_max);
    return str;
  }
}